%{
    #include <string>
    #include <math.h>
    #include <limits>
%}

%require "2.3"
%defines
%skeleton "lalr1.cc"
%name-prefix="DEVS"
%define "parser_class_name" "Parser"
%parse-param { class CDparser *driver }
%locations
%error-verbose

%union
{
    int intVal;
    double doubleVal;
    std::string *stringVal;
}

%token END 0 "end of file"
%token NEWLINE "\n"
%token LBRACKET "(" RBRACKET ")" LCBRACKET "{" RCBRACKET "}" // ( ) { }
%token AND "and" OR "or" XOR "xor" IMP "imp" EQV "eqv" NOT "not"// and or xor imp eqv not
%token TRUE "t" FALSE "f" UNDEFB "?b" UNDEFC "?c" // t f ?
%token PLUS "+" MINUS "-" MUL "*" DIV "/" // + - * /
%token EXCEQV "!=" EQVSIGN "=" GT ">" LT "<" GTEQV ">=" LTEQV "<=" // != = > <  >= <=
%token <stringVal> COND_REAL_FUNC // even odd isInt isPrime isUndefined
%token PORTVALUE "portValue" SEND "send" CELLPOS "cellPos" // portValue send cellPos
%token THISPORT "thisPort" // thisPort
%token IF "if" IFU "ifu" // if ifu
%token <stringVal> CONST_FUNC "const function"
%token <stringVal> WITHOUT_PARAM_FUNC "without parameter function"
%token <stringVal> UNARY_FUNC "unary function"
%token <stringVal> BINARY_FUNC "binary function"
%token COMA "," // ,

%token <intVal> INT
%token <doubleVal> REAL
%token <stringVal> STRING

%left AND OR XOR IMP EQV
%right NOT
%nonassoc EXCEQV EQVSIGN GT LT GTEQV LTEQV
%left PLUS MINUS
%left MUL DIV
%left LBRACKET RBRACKET

%type <doubleVal> CONSTANT BOOL
%type <stringVal> RULELIST RULE RESULT BOOLEXP REALRELEXP REALEXP IDREF FUNCTION CELLREF PORTNAME


%destructor { delete $$; } <stringVal>
//%destructor { std::cerr << "destruktor" << std::endl;delete $$; } CONST_FUNC WITHOUT_PARAM_FUNC UNARY_FUNC BINARY_FUNC

%{
    #include "CDparser.h"
    #undef yylex
    #define yylex driver->scanner->lex
    #define NEWSTR new std::string;
%}

%% // pravidla

START :         RULELIST { driver->parsed = *$1; delete $1; }

RULELIST :      RULE END { $$ = $1; }
                | RULE NEWLINE END { $$ = $1; }
                | RULE NEWLINE RULELIST { $$=NEWSTR; *$$ = *$1 + "\n" + *$3; delete $1; delete $3; }

RULE :          RESULT RESULT LCBRACKET BOOLEXP RCBRACKET { $$=NEWSTR; *$$ = "if("+*$4+"){"+*$1+"##"+*$2+"}"; delete $4; delete $1; delete $2; }

RESULT :        CONSTANT { $$=NEWSTR; if(isnan($1))*$$ = " NAN "; else *$$ = driver->toString($1); }
                | LCBRACKET REALEXP RCBRACKET { $$=NEWSTR; *$$ = "("+*$2+")"; delete $2; }

BOOLEXP :       BOOL { $$=NEWSTR; if(isnan($1))*$$ = " NAN "; else *$$ = driver->toString($1); }
                | LBRACKET BOOLEXP RBRACKET { $$=NEWSTR; *$$ = "("+*$2+")"; delete $2; }
                | REALRELEXP { $$=NEWSTR; *$$ = *$1; delete $1; }
                | NOT BOOLEXP { $$=NEWSTR; *$$ = "!("+*$2+")"; delete $2; }
                | BOOLEXP AND BOOLEXP { $$=NEWSTR; *$$ = *$1+"&&"+*$3; delete $1; delete $3; }
                | BOOLEXP OR BOOLEXP { $$=NEWSTR; *$$ = *$1+"||"+*$3; delete $1; delete $3; }
                | BOOLEXP XOR BOOLEXP { $$=NEWSTR; *$$ = "xor("+*$1+", "+*$3+") "; delete $1; delete $3; }
                | BOOLEXP IMP BOOLEXP { $$=NEWSTR; *$$ = "imp("+*$1+", "+*$3+") "; delete $1; delete $3; }
                | BOOLEXP EQV BOOLEXP { $$=NEWSTR; *$$ = "eqv("+*$1+", "+*$3+") "; delete $1; delete $3; }

REALRELEXP :    REALEXP EXCEQV REALEXP { $$=NEWSTR; *$$ = *$1+"!="+*$3; delete $1; delete $3; }
                | REALEXP EQVSIGN REALEXP { $$=NEWSTR; *$$ = *$1+"="+*$3; delete $1; delete $3; }
                | REALEXP GT REALEXP { $$=NEWSTR; *$$ = *$1+">"+*$3; delete $1; delete $3; }
                | REALEXP LT REALEXP { $$=NEWSTR; *$$ = *$1+"<"+*$3; delete $1; delete $3; }
                | REALEXP GTEQV REALEXP { $$=NEWSTR; *$$ = *$1+">="+*$3; delete $1; delete $3; }
                | REALEXP LTEQV REALEXP { $$=NEWSTR; *$$ = *$1+"<="+*$3; delete $1; delete $3; }
                | COND_REAL_FUNC LBRACKET REALEXP RBRACKET { $$=NEWSTR; *$$ = *$1+"("+*$3+")"; delete $1; delete $3; }

REALEXP :       IDREF { $$=NEWSTR; *$$ = *$1; delete $1; }
                | LBRACKET REALEXP RBRACKET { $$=NEWSTR; *$$ = "("+*$2+")"; delete $2; }
                | REALEXP PLUS REALEXP { $$=NEWSTR; *$$ = *$1+"+"+*$3; delete $1; delete $3; }
                | REALEXP MINUS REALEXP { $$=NEWSTR; *$$ = *$1+"+"+*$3; delete $1; delete $3; }
                | REALEXP MUL REALEXP { $$=NEWSTR; *$$ = *$1+"+"+*$3; delete $1; delete $3; }
                | REALEXP DIV REALEXP { $$=NEWSTR; *$$ = *$1+"+"+*$3; delete $1; delete $3; }

IDREF :         CELLREF { $$=NEWSTR; *$$ = *$1; delete $1; }
                | CONSTANT { $$=NEWSTR; *$$ = driver->toString($1); }
                | FUNCTION { $$=NEWSTR; *$$ = *$1; delete $1; }
                | PORTVALUE LBRACKET PORTNAME RBRACKET { $$=NEWSTR; *$$ = "portValue("+*$3+")"; delete $3; }
                | SEND LBRACKET PORTNAME COMA REALEXP RBRACKET { $$=NEWSTR; *$$ = "send("+*$3+","+*$5+")"; delete $3; delete $5; }
                | CELLPOS LBRACKET REALEXP RBRACKET { $$=NEWSTR; *$$ = "cellPos("+*$3+")"; delete $3; }

CONSTANT :      INT { $$ = $1; }
                | REAL { $$ = $1; }
                | CONST_FUNC { $$ = driver->constFunc($1); delete $1; }
                | UNDEFC { $$ = NAN; }

FUNCTION :      UNARY_FUNC LBRACKET REALEXP RBRACKET { $$=NEWSTR; *$$ = *$1+"("+*$3+")"; delete $1; delete $3; }
                | WITHOUT_PARAM_FUNC { $$=NEWSTR; *$$ = *$1+"()"; delete $1; }
                | BINARY_FUNC LBRACKET REALEXP COMA REALEXP RBRACKET { $$=NEWSTR; *$$ = *$1+"("+*$3+","+*$5+")"; delete $1; delete $3; delete $5; }
                | IF LBRACKET BOOLEXP COMA REALEXP COMA REALEXP RBRACKET { $$=NEWSTR; *$$ = "func_if("+*$3+","+*$5+")"; delete $3; delete $5; delete $7; }
                | IFU LBRACKET BOOLEXP COMA REALEXP COMA REALEXP COMA REALEXP RBRACKET { $$=NEWSTR; *$$ = "func_ifu("+*$3+","+*$5+","+*$7+","+*$9+")"; delete $3; delete $5; delete $7; delete $9; }

CELLREF :       LBRACKET INT COMA INT RBRACKET { $$=NEWSTR; *$$ = "cell_ref("+driver->toString($2)+","+driver->toString($4)+")"; }

/*REST_TUPLA :    COMA INT REST_TUPLA { std::cout << "ZLE" << std::endl; }
                | RBRACKET { std::cout << "ZLE" << std::endl; }*/

BOOL :          TRUE { $$ = 1.0; }
                | FALSE { $$ = 0.0; }
                | UNDEFB { $$ = NAN; }

PORTNAME :      THISPORT { $$=NEWSTR; *$$ = "thisPort"; }
                | STRING { $$=NEWSTR; *$$ = *$1; delete $1; }

%% // dodatocny kod

void DEVS::Parser::error(const Parser::location_type& l, const std::string& m)
{
    std::cerr << "Error " << l.begin.line << ":" << l.begin.column << "-" << l.end.column << " " << m << std::endl;
}
