%{
#include <string>
#include <stdlib.h>
#include "scanner.hh"


typedef DEVS::Parser::token token;
typedef DEVS::Parser::token_type token_type;

//#define yyterminate() return token::END
#define YY_NO_UNISTD_H

#define yyterminate() return token::END

%}

%option c++
%option prefix="Scanner"
%option batch
%option yywrap nounput
%option stack

/* sledovnie polohy parsovania */
%{
#define YY_USER_ACTION  yylloc->columns(yyleng);
%}

%% /* regularne vyrazy */

%{
    yylloc->step();
%}

\%[^\n]*\n              { yylloc->lines(); }
\n                      { yylloc->lines(); return token::NEWLINE; }
<<EOF>>                 { return token::END; }
\{                      { return token::LCBRACKET; }
\}                      { return token::RCBRACKET; }
\(                      { return token::LBRACKET; }
\)                      { return token::RBRACKET; }
[0-9]+                  { yylval->intVal = atoi(YYText()); return token::INT; }
[0-9]+\.[0-9]+          { yylval->doubleVal = atof(YYText()); return token::REAL; }
t                       { return token::TRUE; }
f                       { return token::FALSE; }
\?b                     { return token::UNDEFB; }
\?c                     { return token::UNDEFC; }
\+                      { return token::PLUS; }
\-                      { return token::MINUS; }
\*                      { return token::MUL; }
\/                      { return token::DIV; }
"!="                    { return token::EXCEQV; }
"="                     { return token::EQVSIGN; }
">"                     { return token::GT; }
"<"                     { return token::LT; }
">="                    { return token::GTEQV; }
"<="                    { return token::LTEQV; }
","                     { return token::COMA; }
"and"                   { return token::AND; }
"or"                    { return token::OR; }
"xor"                   { return token::XOR; }
"imp"                   { return token::IMP; }
"eqv"                   { return token::EQV; }
(even|odd|isInt|isPrime|isUndefined) {
                        yylval->stringVal = new std::string(YYText()); return token::COND_REAL_FUNC; }
(pi|e|inf|grav|accel|light|planck|avogadro|faraday|rydberg|euler_gamma|bohr_radius|boltzmann|bohr_magneton|golden|catalan|amu|electron_charge|ideal_gas|stefan_boltzmann|proton_mass|electron_mass|neutron_mass|pem) {
                        yylval->stringVal = new std::string(YYText()); return token::CONST_FUNC; }
(truecount|falsecount|undefcount|time|random|randomSign) {
                        yylval->stringVal = new std::string(YYText()); return token::WITHOUT_PARAM_FUNC; }
(abs|acos|acosh|asin|asinh|atan|atanh|cos|sec|sech|exp|cosh|fact|fractional|ln|log|round|cotan|cosec|cosech|sign|sin|sinh|statecount|sqrt|tan|tanh|trunc|truncUpper|poisson|exponential|randInt|chi|asec|acotan|asech|acosech|nextPrime|radToDeg|degToRad|nth_prime|acotanh|CtoF|CtoK|KtoC|KtoF|FtoC|FtoK) {
                        yylval->stringVal = new std::string(YYText()); return token::UNARY_FUNC; }
(comb|logn|max|min|power|remainder|root|beta|gamma|lcm|gcd|normal|f|uniform|binomial|rectToPolar_r|rectToPolar_angle|polarToRect_x|hip|polarToRect_y) {
                        yylval->stringVal = new std::string(YYText()); return token::BINARY_FUNC; }
"portValue"             { return token::PORTVALUE; }
"send"                  { return token::SEND; }
"cellPos"               { return token::CELLPOS; }
"thisPort"              { return token::THISPORT; }
"ifu"                   { return token::IFU; }
"if"                    { return token::IF; }
[A-Za-z]+               { yylval->stringVal = new std::string(YYText()); return token::STRING; }
.                       { }

%% /* dodatocny kod */

int ScannerFlexLexer::yylex()
{
    std::cerr << "in ExampleFlexLexer::yylex() !" << std::endl;
    return 0;
}

int ScannerFlexLexer::yywrap()
{
    return 1;
}
