#include "glwidget.h"
#include <iostream>
#include <QPixmap>
#include <QImage>
#include <QMouseEvent>
#include <QMdiSubWindow>
#include <QFile>
#include "cellspace.h"
#include "model.h"
#include "machine.h"
#include "gpuperf/include/gpaload.h"

/**
@param format format ktory sa ma pouzit pri vytvarani OpenGL okna. pozri Qt dokumentaciu ku QGLWidget
@param parent Qt rodic
@param space model ktory sa priradi aktualnemu oknu
*/
GLwidget::GLwidget(const QGLFormat &format, QWidget *parent, CellSpace *space) : QGLWidget(format, parent)
{
    pos_x = pos_y = 0;
    zoom = 1;
    this->space = space;
    if(space)connect(&timer, SIGNAL(timeout()), this, SLOT(runSimulation()));
    spaceName = trUtf8("Nový stav ")+QString::number(getUntitledNumber());
}

GLwidget::~GLwidget()
{
    //if(space)delete space;
    //makeCurrent();
    //glDeleteTextures(1, &tex);
}
/**
Inicializacia OpenGL
*/
void GLwidget::initializeGL()
{
    //glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glGenTextures(1, &tex);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glEnable(GL_TEXTURE_2D);
    gpa_uint32 num_counters,num_passes;
    if(!initialized)
    {
        std::cout << "Inicializujem OpenGL" << std::endl;
        std::cout << "\tVendor: \t" << glGetString(GL_VENDOR) << std::endl;
        std::cout << "\tRenderer:\t" << glGetString(GL_RENDERER) << std::endl;
        std::cout << "\tVersion:\t" << glGetString(GL_VERSION) << std::endl;
        //std::cout << "\tGLSL version:\t" << glGetString(GL_SHADING_LANGUAGE_VERSION) << std::endl;
        //std::cout << getGPAError(GPA_OpenContext(glXGetCurrentContext())) << endl;
        //std::cout << getGPAError(GPA_GetNumCounters(&num_counters));
        //GPA_EnableAllCounters();
        //cout << " " << num_counters << endl;
        //char *name;
        /*for(unsigned int i=0;i<num_counters;i++)
        {
            GPA_GetCounterName(i, (const char**)&name);
            cout << name << endl;
        }*/
        //GPA_EnableCounter(1);
        //GPA_GetPassCount(&num_passes);
        //cout << "Pocet prechodov: " << num_passes << endl;
        initialized = true;
    }
    if(space)
    {
        unsigned char *tex = space->renderToTexture();
        loadTexture(space->getWidth(), space->getHeight(), tex);
        delete [] tex;
    }
    //updateGL();
}
/**
Vykreslovanie OpenGL sceny
*/
void GLwidget::paintGL()
{
    /*gpa_uint32 sessionID;
    GPA_BeginSession(&sessionID);

    GPA_BeginPass();
    GPA_BeginSample(0);*/

    glClear(GL_COLOR_BUFFER_BIT);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex);
    glBegin(GL_QUADS);
    //glColor3f(1.0f, 0.0f, 0.0f);
    /*glTexCoord2d(0.0f, 0.0f);glVertex3f(-1.0f, -1.0f, 0.0f);
    glTexCoord2d(1.0f, 0.0f);glVertex3f( 1.0f, -1.0f, 0.0f);
    glTexCoord2d(1.0f, 1.0f);glVertex3f( 1.0f,  1.0f, 0.0f);
    glTexCoord2d(0.0f, 1.0f);glVertex3f(-1.0f,  1.0f, 0.0f);*/
    glTexCoord2d((float)pos_x/tex_w, (float)pos_y/tex_h);glVertex3f(-1.0f, -1.0f, 0.0f);
    glTexCoord2d((float)(pos_x+w*zoom)/tex_w, (float)pos_y/tex_h);glVertex3f( 1.0f, -1.0f, 0.0f);
    glTexCoord2d((float)(pos_x+w*zoom)/tex_w, (float)(pos_y+h*zoom)/tex_h);glVertex3f( 1.0f,  1.0f, 0.0f);
    glTexCoord2d((float)pos_x/tex_w, (float)(pos_y+h*zoom)/tex_h);glVertex3f(-1.0f,  1.0f, 0.0f);
    glEnd();

    /*GPA_EndSample();
    GPA_EndPass();
    GPA_EndSession();*/
    glFinish();

    /*GPA_Status sessionStatus;
    bool readyResult;
    sessionStatus = GPA_IsSessionReady( &readyResult, sessionID);
    cout << "Status " << getGPAError(sessionStatus) << " " << readyResult << endl;

    GPA_Type type;
    GPA_GetCounterDataType(0, &type);
    if ( type == GPA_TYPE_UINT32 )
    {
        gpa_uint32 value=0;
        cout<<getGPAError(GPA_GetSampleUInt32(sessionID, 0, 1, &value))<< endl;;
        cout << value << endl;
    }
    else if ( type == GPA_TYPE_UINT64 )
    {
        gpa_uint64 value=0;
        cout<<getGPAError(GPA_GetSampleUInt64(sessionID, 0, 1, &value))<< endl;;
        cout << value << endl;
    }
    else if ( type == GPA_TYPE_FLOAT32 )
    {
        gpa_float32 value=0;
        cout<<getGPAError(GPA_GetSampleFloat32(sessionID, 0, 1, &value))<< endl;;
        cout << value << endl;
    }
    else if ( type == GPA_TYPE_FLOAT64 )
    {
        gpa_float64 value=0;
        cout<<getGPAError(GPA_GetSampleFloat64(sessionID, 0, 1, &value))<< endl;;
        cout << value << endl;
    }*/
}

void GLwidget::resizeGL(int w,int h)
{
    glViewport(0, 0, w, h);
    this->w = w;
    this->h = h;
}
/**
Nacita texturu do OpenGL do akktualneho widgetu
@param w sirka textury
@param h vyska textury
@param texture pixelove data textury
*/
void GLwidget::loadTexture(int w, int h, unsigned char *texture)
{
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, texture);
    tex_w = w;
    tex_h = h;
}

void GLwidget::mouseMoveEvent(QMouseEvent *event)
{
    pos_x -= (event->x() - mouse_x)*zoom;
    pos_y += (event->y() - mouse_y)*zoom;
    mouse_x = event->x();
    mouse_y = event->y();
    updateGL();
}

void GLwidget::mousePressEvent(QMouseEvent *event)
{
    mouse_x = event->x();
    mouse_y = event->y();
}

void GLwidget::wheelEvent(QWheelEvent *event)
{

    if(event->delta() > 0)//zoom in
    {
        zoom *= 2;
        /*pos_x = pos_x + w/4;
        pos_y = pos_y + h/4;*/
    }
    else//zoom out
    {
        if(zoom <= 1/16.f)return;//nad sestnastnasobne zvetsenie nejdem
        zoom /= 2;
        /*pos_x = pos_x - w/2;
        pos_y = pos_y - h/2;*/
    }
    updateGL();
}

bool GLwidget::initialized = false;
int GLwidget::number = 1;

void GLwidget::startSimulation()
{
    timer.start(1000);
}

void GLwidget::stopSimulation()
{
    timer.stop();
}

void GLwidget::runSimulation()
{
    space->run(1);
    space->getParent()->getParent()->finish();
    unsigned char *tex = space->renderToTexture();
    makeCurrent();
    loadTexture(space->getWidth(), space->getHeight(), tex);
    delete [] tex;
    updateGL();
}

void GLwidget::setSubWindowTitle()
{
    QMdiSubWindow *parent = dynamic_cast<QMdiSubWindow*>(parentWidget());
    if(parent)parent->setWindowTitle(spaceName);
}

int GLwidget::getUntitledNumber()
{
    return number++;
}

int GLwidget::load(const QString &file)
{
    QFile fr(file);
    if(!fr.exists())return fr.error();
    if(!fr.open(QIODevice::ReadOnly | QIODevice::Text))return fr.error();
    return 0;
}

int GLwidget::save(const QString &file)
{
    return 0;
}

QString GLwidget::getFileName()
{
    return spaceName;
}
