#include "texteditor.h"
#include <QMdiSubWindow>

Hightligter::Hightligter(QTextDocument *document) : QSyntaxHighlighter(document)
{
    HightlightingRule rule;
    //puncations {} :
    rule.pattern = QRegExp("[\\{\\}:<>=\\(\\)]");
    rule.format = QTextCharFormat();
    rule.format.setForeground(Qt::red);
    rules.append(rule);
    //number
    rule.pattern = QRegExp("[0123456789]");
    rule.format = QTextCharFormat();
    rule.format.setForeground(Qt::magenta);
    rules.append(rule);
    //keywords
    rule.pattern = QRegExp("\\b(rule|and|or|xor|imp|eqv|t|f|\\?)\\b");
    rule.format = QTextCharFormat();
    rule.format.setForeground(Qt::darkBlue);
    rule.format.setFontWeight(QFont::Bold);
    rules.append(rule);
    //section [top]
    rule.pattern = QRegExp("\\[[a-zA-Z_\\-0-9]*\\]");
    rule.format = QTextCharFormat();
    rule.format.setFontWeight(QFont::Bold);
    rule.format.setForeground(Qt::green);
    rules.append(rule);
    //comments %
    rule.pattern = QRegExp("%.*");
    rule.format = QTextCharFormat();
    rule.format.setFontItalic(true);
    rule.format.setForeground(Qt::gray);
    rules.append(rule);
}

void Hightligter::highlightBlock(const QString &text)
{
    for(QVector<HightlightingRule>::iterator i = rules.begin();i != rules.end();i++)
    {
         QRegExp expression((*i).pattern);
         int index = expression.indexIn(text);
         while (index >= 0)
         {
             int length = expression.matchedLength();
             setFormat(index, length, (*i).format);
             index = expression.indexIn(text, index + length);
         }
     }
     setCurrentBlockState(0);
}

int TextEditor::number = 1;

TextEditor::TextEditor(const QString &text, QWidget *parent)
{
    document.setPlainText(text);
    highlighter = new Hightligter(&document);
    setDocument(&document);
    QFont font;
    font.setFamily("Courier");
    font.setFixedPitch(true);
    font.setPointSize(10);
    setFont(font);
    setAcceptRichText(false);
    setLineWrapMode(QTextEdit::NoWrap);
    modelName = trUtf8("Nový model ")+getUntitledNumber();
    setMinimumSize(400, 300);
}

TextEditor::~TextEditor()
{
    delete highlighter;
}

void TextEditor::setName(const QString &name)
{
    modelName = name;
    QMdiSubWindow *parent = dynamic_cast<QMdiSubWindow*>(parentWidget());
    if(parent)parent->setWindowTitle(modelName);
}

void TextEditor::setSubWindowTitle()
{
    QMdiSubWindow *parent = dynamic_cast<QMdiSubWindow*>(parentWidget());
    if(parent)parent->setWindowTitle(modelName);
}

int TextEditor::load(const QString &file)
{

}

int TextEditor::save(const QString &file)
{

}

QString TextEditor::getFileName()
{
    return modelName;
}
